BeginPackage["RSA`"]

RSAEncode::usage="RSAEncode[s, e, n] turns the message-string s into a number and then encodes that number by the RSA method using exponent e and modulus n. If s is too large, then a list of integers is returned.";

RSADecode::usage="RSADecode[m, k, n] decodes the integer (or list) m using the decoding inverse k and the modulus n.";

Begin["`Private`"]

StringToInteger[m_String]:=FromDigits[ToCharacterCode[m],65536]

IntegerToString[m_]:=FromCharacterCode[IntegerDigits[m,65536]]

RSAEncode[m_String,e_Integer,n_Integer]:=PowerMod[StringToInteger[m],e,n]/;StringLength[m]<=Log[65536,n]

RSAEncode[m_String,e_Integer,n_Integer]:=Module[{maxlength=Floor[Log[65536,n]],length=StringLength[m]},
Map[RSAEncode[#,e,n]&,Table[StringTake[m,{pos,Min[length,pos+maxlength-1]}],{pos,1,length,maxlength}]]]/;StringLength[m]>Log[65536,n]

RSADecode[m_Integer,d_Integer,n_Integer]:=IntegerToString[PowerMod[m,d,n]]

RSADecode[m_List,d_Integer,n_Integer]:=StringJoin@@Map[RSADecode[#,d,n]&,m]

End[]

EndPackage[]